import unittest
import os

from kumaoche.config import ConfigParser


class TestConfigParser(unittest.TestCase):
    def setUp(self):
        self.file_path_list = [os.path.dirname(__file__) + '/stub_config.yml']

    def test_all_roles(self):
        self.assertEqual(['variable_assign_test_role', 'empty_role'], ConfigParser.all_repository_names(self.file_path_list))

    def test_find(self):
        config = ConfigParser.find('variable_assign_test_role', self.file_path_list)
        self.assertEqual(config.services[0].lang, 'git')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from .config import ConfigParser, ServiceConfig
from .exec_env import DisableEnv, Shell, Docker, StringBuilder
from .service import PackageManager
from .runner import InvokeRunner


class Container(object):

    @staticmethod
    def env(config: ServiceConfig):
        if config.env == 'string_builder':
            return StringBuilder(config.string_builder, config.environment, InvokeRunner)
        elif config.env == 'shell':
            return Shell(config.shell, config.environment, InvokeRunner)
        elif config.env == 'docker':
            return Docker(config.docker, config.environment, InvokeRunner)
        else:
            return DisableEnv()

    @classmethod
    def all_repository_names(cls):
        return ConfigParser.all_repository_names()

    @classmethod
    def find_services(cls, repository_name: str):
        services = []

        for config in ConfigParser.find(repository_name):
            for service in config.services:
                services.extend([PackageManager(cls.env(service), service)])

        return services

    @classmethod
    def find_services_by_name(cls, repository_name: str, service_lang_name: str):
        services = []

        for config in ConfigParser.find(repository_name):
            for service in config.services:
                if service.name() == service_lang_name:
                    services.extend([PackageManager(cls.env(service), service)])

        return services

    @classmethod
    def find_all_services(cls):
        services = []

        for role in ConfigParser.all_repository_names():
            for config in ConfigParser.find(role):
                for service in config.services:
                    services.extend([PackageManager(cls.env(service), service)])

        return services

    @classmethod
    def find_all_services_by_name(cls, service_lang_name: str):
        services = []

        for role in ConfigParser.all_repository_names():
            for config in ConfigParser.find(role):
                for service in config.services:
                    if service.name() == service_lang_name:
                        services.extend([PackageManager(cls.env(service), service)])

        return services



# -*- coding: utf-8 -*-


class ExecEnvConfig(object):
    def __init__(self, parsed_yaml: {}, key: str):
        configs = parsed_yaml.get(key, {})

        self.working_dir = configs.get('working_dir', '')
        self.run = configs.get('run', '')
        self.build = configs.get('build', '')
        self.up = configs.get('up', '')
        self.down = configs.get('down', '')



import unittest
from kumaoche.config import ConfigPack


class TestConfigPack(unittest.TestCase):
    def test_init(self):
        parsed_yaml = {
            'presets': {'environments': {'key': 'bar', 'foo': 'bar'}},
            'environments': {'key': 'value'},
            'string_builder': {'name': '1'},
            'shell': {'name': '2'},
            'docker': {'name': '3'},
            'git': {'run': '100'},
            'php': {'run': '101'},
            'ruby': {'run': '102'},
            'node': {'run': '103'},
        }
        config = ConfigPack(parsed_yaml)
        # self.assertEqual({'key': 'value', 'foo': 'bar'}, config.variable)
        # self.assertEqual('1', config.string_builder.name)
        # self.assertEqual('2', config.shell.name)
        # self.assertEqual('3', config.docker.name)
        # self.assertEqual('100', config.git.run)
        # self.assertEqual('101', config.php.run)
        # self.assertEqual('102', config.ruby.run)
        # self.assertEqual('103', config.node.run)

        config = ConfigPack({})
        # self.assertEqual({}, config.variable)
        # self.assertEqual('', config.string_builder.name)
        # self.assertEqual('', config.shell.name)
        # self.assertEqual('', config.docker.name)
        # self.assertEqual('', config.git.run)
        # self.assertEqual('', config.php.run)
        # self.assertEqual('', config.ruby.run)
        # self.assertEqual('', config.node.run)


if __name__ == '__main__':
    unittest.main()



import unittest
from kumaoche.config import StringBuilderConfig


class TestStringBuilderConfig(unittest.TestCase):
    def test_init(self):
        parsed_yaml = {
            'string_builder': {
                'run': '0',
                'name': '1',
                'working_dir': '2',
                'build': '5',
                'up': '6',
                'down': '7',
            }
        }
        config = StringBuilderConfig(parsed_yaml, 'string_builder')
        self.assertEqual('0', config.run)
        self.assertEqual('2', config.working_dir)
        self.assertEqual('5', config.build)
        self.assertEqual('6', config.up)
        self.assertEqual('7', config.down)

        config = StringBuilderConfig({}, 'string_builder')
        self.assertEqual('', config.run)
        self.assertEqual('', config.working_dir)
        self.assertEqual('', config.build)
        self.assertEqual('', config.up)
        self.assertEqual('', config.down)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from ..exec_env import ShellConfig, DockerConfig, StringBuilderConfig


class ServiceConfig(object):
    def __init__(self, configs: {}):

        self.lang = configs.get('lang', '')
        self.env = configs.get('env', '')
        self.environment = configs.get('environment', '')
        self.string_builder = StringBuilderConfig(configs, 'string_builder')
        self.shell = ShellConfig(configs, 'shell')
        self.docker = DockerConfig(configs, 'docker')

        # service 毎の各言語用コマンドは個別設定を優先する
        lang_config = configs.get(self.lang, {})
        self.run = configs.get('run', lang_config.get('run', ''))
        self.setup = configs.get('setup', lang_config.get('setup', ''))
        self.update = configs.get('update', lang_config.get('update', ''))



# -*- coding: utf-8 -*-

from abc import ABCMeta, abstractmethod


class ExecEnv(metaclass=ABCMeta):
    @abstractmethod
    def name(self):
        raise NotImplementedError()

    @abstractmethod
    def var_assign(self, text: str, append_var=None):
        raise NotImplementedError()

    @abstractmethod
    def build(self, command: str, work_dir=''):
        raise NotImplementedError()

    @abstractmethod
    def up(self, command: str, work_dir=''):
        raise NotImplementedError()

    @abstractmethod
    def down(self, command: str, work_dir=''):
        raise NotImplementedError()

    @abstractmethod
    def run(self, command: str, work_dir=''):
        raise NotImplementedError()



# -*- coding: utf-8 -*-

from .service import PackageManagerConfig


class ConfigPack(object):
    def __init__(self, parsed_yaml):
        if parsed_yaml is None:
            parsed_yaml = {}

        # 全設定共有のテンプレート用変数
        self.environment = self.assign_dict(parsed_yaml, 'environment')
        self.services = []

        git = parsed_yaml.get('git', {})
        if git is not {}:
            git['environment'] = self.environment
            self.services.extend([PackageManagerConfig(git)])

        for service in parsed_yaml.get('services', []):
            if service is not None:
                self.services.extend([PackageManagerConfig(service)])

    @classmethod
    def assign_dict(cls, parsed_yaml: {}, key: str):
        configs = parsed_yaml.get(key, {})
        presets_configs = parsed_yaml.get('presets', {}).get(key, {})

        dictionary = {}
        for key in configs.keys():
            dictionary.setdefault(key, configs.get(key, ''))

        for key in presets_configs.keys():
            dictionary.setdefault(key, presets_configs.get(key, ''))

        return dictionary



# -*- coding: utf-8 -*-

from .config_parser import ConfigParser
from .config_pack import ConfigPack
from .exec_env import ExecEnvConfig, DockerConfig, ShellConfig, StringBuilderConfig
from .service import ServiceConfig, PackageManagerConfig



import unittest
from kumaoche.config import PackageManagerConfig


class TestPackageManagerConfig(unittest.TestCase):
    def test_init(self):
        parsed_yaml = {
            'lang': 'php',
            'env': 'docker',
            'php': {
                'run': '0',
                'setup': '1',
                'update': '2',
                'test': '3',
            },
            'node': {
                'run': '10',
                'setup': '11',
                'update': '12',
                'test': '13',
            }
        }

        config = PackageManagerConfig(parsed_yaml)
        self.assertEqual('0', config.run)
        self.assertEqual('1', config.setup)
        self.assertEqual('2', config.update)
        self.assertEqual('3', config.test)

        config = PackageManagerConfig({})
        self.assertEqual('', config.run)
        self.assertEqual('', config.setup)
        self.assertEqual('', config.update)
        self.assertEqual('', config.test)


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='kumaoche',
    version='0.1.22',
    description='Pythonic Dev Repository & Dev Environment Management tool.',
    long_description='',
    author='kumak1',
    author_email='kumaki0@gmail.com',
    install_requires=['invoke', 'pyyaml'],
    url='https://github.com/kumak1/kumaoche',
    license=license,
    packages=find_packages(exclude=('tests', 'docs', 'examples')),
    test_suite='tests'
)
